@@ -40,36 +40,58 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.5.4.8.js';
+gTestfile = '13.5.1.js';
 
-START("13.5.4.8 - XMLList copy()");
+START("13.5.1 - XMLList Constructor as Function");
 
-TEST(1, true, XMLList.prototype.hasOwnProperty("copy"));
+x = XMLList();
 
-emps = new XMLList();
-emps += <employee id="0"><name>Jim</name><age>25</age></employee>;
-emps += <employee id="1"><name>Joe</name><age>20</age></employee>;
+TEST(1, "xml", typeof(x));  
+TEST(2, true, x instanceof XMLList);
 
+// Make sure it's not copied if it's an XMLList
+x =
+<>
+    <alpha>one</alpha>
+    <bravo>two</bravo>
+</>;
+
+y = XMLList(x);
+TEST(3, x === y, true);
+
+x += <charlie>three</charlie>;
+TEST(4, x === y, false);
+
+// Load from one XML type
+x = XMLList(<alpha>one</alpha>);
+TEST_XML(5, "<alpha>one</alpha>", x);
+
+// Load from Anonymous
+x = XMLList(<><alpha>one</alpha><bravo>two</bravo></>);
 correct = new XMLList();
-correct += <employee id="0"><name>Jim</name><age>25</age></employee>;
-correct += <employee id="1"><name>Joe</name><age>20</age></employee>;
+correct += <alpha>one</alpha>;
+correct += <bravo>two</bravo>;
+TEST_XML(6, correct.toString(), x);
 
-TEST(2, emps, emps.copy());
-TEST(3, correct, emps.copy());
-
-// Make sure we're getting a copy, not a ref to orig.
-emps = new XMLList();
-emps += <employee id="0"><name>Jim</name><age>25</age></employee>;
-emps += <employee id="1"><name>Joe</name><age>20</age></employee>;
-  
+// Load from Anonymous as string
+x = XMLList(<><alpha>one</alpha><bravo>two</bravo></>);
 correct = new XMLList();
-correct += <employee id="0"><name>Jim</name><age>25</age></employee>;
-correct += <employee id="1"><name>Joe</name><age>20</age></employee>;
+correct += <alpha>one</alpha>;
+correct += <bravo>two</bravo>;
+TEST_XML(7, correct.toString(), x);
 
-x = emps.copy();
+// Load from single textnode
+x = XMLList("foobar");
+TEST_XML(8, "foobar", x);
 
-emps += <employee id="2"><name>Sue</name><age>32</age></employee>;
+x = XMLList(7);
+TEST_XML(9, "7", x);
 
-TEST(4, correct, x);
+// Undefined and null should behave like ""
+x = XMLList(null);
+TEST_XML(10, "", x);
+
+x = XMLList(undefined);
+TEST_XML(11, "", x);
 
 END();
\ No newline at end of file
