@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2007
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Igor Bukanov
+ * Contributor(s): Jesse Ruderman
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -35,16 +35,29 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-var gTestfile = 'regress-407727-01.js';
+var gTestfile = 'regress-466905-01.js';
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 407727;
-var summary = 'let Object global redeclaration';
+var BUGNUMBER = 466905;
+var summary = 'Do not assert: v_ins->isCall() && v_ins->callInfo() == &js_FastNewArray_ci';
 var actual = '';
-var expect = 1;
+var expect = '';
 
-printBugNumber(BUGNUMBER);
-printStatus (summary);
+
+//-----------------------------------------------------------------------------
+test();
+//-----------------------------------------------------------------------------
+
+function test()
+{
+  enterFunc ('test');
+  printBugNumber(BUGNUMBER);
+  printStatus (summary);
  
-let Object = 1;
-actual = Object;
-reportCompare(expect, actual, summary);
+  function f(a) { for each (let c in a) [(c > 5) ? 'A' : 'B']; }
+  f([true, 8]);
+  f([2]);
+
+  reportCompare(expect, actual, summary);
+
+  exitFunc ('test');
+}
